/***************************************************************************
                          CHAT_Manager.cc  -  description
                             -------------------
    begin                : Thu Jun 28 2001
    copyright            : (C) 2001 by 
    email                : ganda_kusuma@usa.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "CHAT_Manager.hh"
#include "SOCK_Socket.hh"

// ugly global ptr variables
static SOCK_Socket *__serverSocketPtr=NULL;
static CHAT_Room   *__lobbyRoomPtr=NULL;


static void
sigint_handler(int sig)
{
  printf("Manager: SigintHandler: Closing connection...!\n");

  switch(sig) {
  case SIGPIPE:
    printf ("A session is disconnected by remote peer.\n");
    break;
  default:
    if (__lobbyRoomPtr != NULL){
      ((CHAT_Room*)__lobbyRoomPtr)->DisconnectAll();
    }
    if (__serverSocketPtr != NULL ){
      delete (__serverSocketPtr);
    }
    exit(1);
    break;
  }
}

CHAT_Manager::CHAT_Manager()
{
  int i;

  for (i=0;i<MAX_ROOM;i++)
    roomsPtrCv[i] = NULL;
}


CHAT_Manager::~CHAT_Manager(){
}

int CHAT_Manager::Begin()
{
  CHAT_Room      lobby;   // only for cleaning up
  CHAT_Login     login;   // database user login
  CHAT_User     *userPtr; // thread & user
  SOCK_Socket   *clientSocketPtr;

  __lobbyRoomPtr = &lobby;

  Boolean     serverFlag = TRUE;
  Word        port = 5000;
  int         pid;

  strcpy(lobby.roomNameCv, "_LOBBY_");

  if (signal(SIGINT, sigint_handler) == SIG_ERR) {
    printf ("MANAGER: ERROR: Cannot create a sigint handler.\n");
    exit(1);
  }

  if (signal(SIGPIPE, sigint_handler) == SIG_ERR) {
    printf ("MANAGER: ERROR: Cannot create a sigpipe handller\n");
    exit(1);
  }

  __serverSocketPtr = new SOCK_Socket(serverFlag);

  if (__serverSocketPtr == NULL) {
    printf ("MANAGER: ERROR: Cannot create a server socket.\n");
    return (-1);
  }

  if (ERROR == __serverSocketPtr->BindAndListen(port)) {
    printf ("MANAGER: ERROR: Cannot bind and listen at port %d\n", port);
    return (-1);
  }

  printf ("MANAGER : Server is listenning at port %d\n", port);
  printf ("MANAGER : Enter ctrl-break to close..\n");
  while(1) {
    clientSocketPtr = __serverSocketPtr->Accept();
    printf ("MANAGER : Accepting a new user.\n");
    userPtr = new CHAT_User(clientSocketPtr);
    if (userPtr == NULL) {
      printf("MANAGER : Error memory allocation\n");
    };
    userPtr->SetManager(this);
    userPtr->SetLogin(&login);
    userPtr->StartReceive();
    login.DisplayMotd (userPtr);
    lobby.Join(userPtr);
    userPtr->Start();
    sleep(1);
    userPtr->GetPID(&pid);
//
    printf ("MANAGER : A New Process has started %d\n", pid);
  }
  return EXIT_SUCCESS;
}

Status
CHAT_Manager::RoomRegister (CHAT_Room * roomPtr)
{
  int i;

  for (i=0;i<MAX_ROOM;i++) {
    if (roomsPtrCv[i] == NULL) {
       roomsPtrCv[i] = roomPtr;
       roomsCountCv ++;
       return(OK);
    }
  }
  return(ERROR);
}

Boolean
CHAT_Manager::IsRoomIn (char * roomNamePtr)
{
  int i;
  for (i=0;i<MAX_ROOM;i++) {
    if (roomsPtrCv[i] != NULL) {
       if (!(strcmp(roomsPtrCv[i]->roomNameCv,roomNamePtr))){
//         printf ("Room found!: %d,%p\n", i,roomsPtrCv[i]);
         return (TRUE);
       }
    }
  }
  return (FALSE);
}

CHAT_Room *
CHAT_Manager::GetRoomPtrByName(char * roomName)
{
  int i;
  for (i=0;MAX_ROOM;i++) {
//  printf ("i = %d, ptr = %p\n", i,roomsPtrCv[i]);
    if (roomsPtrCv[i] != NULL) {
      if (!(strcmp(roomsPtrCv[i]->roomNameCv,roomName))){
//        printf ("Room found! %d,%p\n", i,roomsPtrCv[i]);
        return (roomsPtrCv[i]);
      }
    }
  }
  return (NULL);
}


Status
CHAT_Manager::DisplayAllRoomName (CHAT_User * userPtr)
{
  int i;
  Status stat=ERROR;
  for (i=0;i<MAX_ROOM;i++) {
    if (roomsPtrCv[i] != NULL) {
      userPtr->SendMessage (roomsPtrCv[i]->roomNameCv);
      userPtr->SendMessage (" ");
      stat=OK;
    }
  }
  return (stat);
}