#include "SOCK_Socket.hh"
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <arpa/inet.h>
 
/*********************************************************************
*  SOCK_Socket::SOCK_Socket - Constructor
*
* DESCRIPTION:
*   serverflagCV TRUE = Server, FALSE = client;
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
SOCK_Socket::SOCK_Socket (Boolean serverFlag) 
{
  ipAddressCv[0] = '\0';
  serverFlagCv = serverFlag;
  connectedFlagCv = FALSE;

  /* initialize the rest */
  bzero ((char *) &sinCv, sizeof(sinCv));
  sinCv.sin_family = AF_INET;

  /* 6 means TCP */
  sinFdCv = socket (AF_INET, SOCK_STREAM, 6);
}

/*********************************************************************
*  SOCK_Socket::SOCK_Socket - Constructor
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
SOCK_Socket::SOCK_Socket(int clientFd, struct sockaddr_in *clientSinPtr)
{
  char *str;

  serverFlagCv = FALSE;
  sinFdCv = clientFd;
  sinCv   = *clientSinPtr;

  str = inet_ntoa(sinCv.sin_addr);
  strcpy(ipAddressCv,str);

  connectedFlagCv = TRUE;
}

/*********************************************************************
*  SOCK_Socket::~SOCK_Socket - Destructor
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
SOCK_Socket::~SOCK_Socket() 
{
  printf("SOCK: Disconnecting..  \n");
  if (sinFdCv != 0)
    Disconnect();
}

/*********************************************************************
* SOCK_Socket::BindAndListen - 
*
* DESCRIPTION:
*   This is part of Socket server.
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
Status
SOCK_Socket::BindAndListen(Word port /* in: */, Word backlog /* in: */)
{
  int rc;

  if (serverFlagCv == FALSE) {
    printf ("ERROR: This is not a Server Socket.\n");
    return (ERROR);
  }

  sinCv.sin_addr.s_addr = htonl(INADDR_ANY); /* accept all */
  sinCv.sin_port = htons(port);
  rc = bind(sinFdCv, (struct sockaddr *)&sinCv, sizeof(sinCv));

  if (rc ==  -1) {
    printf ("ERROR: Cannot bind to socket.\n");
    return (ERROR);
  }
  
  rc = listen (sinFdCv, backlog);
  if (rc == -1) {
    printf ("ERROR: Cannot listen to socket.\n");
    return (ERROR);
  }
  
  connectedFlagCv = TRUE;
  return (OK);
}

/*********************************************************************
*  SOCK_Socket::Accept - 
*
* DESCRIPTION:
*   This is part of Socket Server
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
SOCK_Socket *
SOCK_Socket::Accept(void)
{
  int                clientFd;
  struct sockaddr_in clientSin;
  socklen_t          length;

  length = sizeof(clientSin);
  clientFd = accept(sinFdCv, (struct sockaddr *)&clientSin, &length);
    
  if (clientFd !=  -1) {
    return (new SOCK_Socket(clientFd, &clientSin));
  }

  return (NULL);
}

/*********************************************************************
*  SOCK_Socket::Connect - 
*
* DESCRIPTION:
*   This is part of Socket client
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
Status
SOCK_Socket::Connect(char *ipAddress /* in: */, Word port /* in: */) 
{
  long ip;
  if (serverFlagCv) {
    return (ERROR);
  }

  strcpy (ipAddressCv, ipAddress);
  if ((ip = inet_addr(ipAddressCv)) == -1) {
    printf ("ERROR: Invalid IP Address.\n");
    return (ERROR);
  }
  
  sinCv.sin_addr.s_addr = htonl(ip);
  sinCv.sin_port = htons(port);

  if (connect (sinFdCv, (struct sockaddr *)&sinCv, sizeof(sinCv)) == 0) {
    connectedFlagCv = TRUE;
    return (OK);
  }

  return (ERROR);
}

/*********************************************************************
*  SOCK_Socket::Disconnect - 
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
Status
SOCK_Socket::Disconnect()
{
  if (connectedFlagCv) {
    close(sinFdCv);
    sinFdCv = 0;
  }

  return (OK);
}

/*********************************************************************
*  SOCK_Socket::GetChar - 
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
Status
SOCK_Socket::GetChar(char &ch /* out: */)
{
  int  count;
  
  count = read(sinFdCv, &ch, 1);

  if (count == 0) {
    return (EOF);
  } else if (count > 0) {
    return (OK);
  }
    
  return (ERROR);
}

int
SOCK_Socket::GetChar(void) 
{
  int  count;
  int  ch;
  count = read(sinFdCv, &ch, 1);

  if (count <= 0) {
    return (-1);
  }

  return (ch);
}


/*********************************************************************
*  SOCK_Socket::PutChar - 
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
Status
SOCK_Socket::PutChar(char ch)
{
  if (write(sinFdCv, &ch, 1) > 0) {
    return (OK);
  }
  return (ERROR);
}

/*********************************************************************
*  SOCK_Socket::PutString - 
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.17 gsusilo: Creation
**********************************************************************/
Status
SOCK_Socket::PutString(char *str)
{
  int length;
  length = strlen(str);
  if (write(sinFdCv, str, length) > 0) {
    return (OK);
  }
  return (ERROR);
}


/*********************************************************************
*  SOCK_Socket::GetString -
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.25 ganda: Creation
**********************************************************************/
Status
SOCK_Socket::GetString(char *buffer)
{
  int i;

  i=recv(sinFdCv, buffer, 50,0);
  if (i==-1){
    return (ERROR);
  } else {
    buffer [i] ='\0'; //add termination string
    return(OK);
  }
}

/*********************************************************************
*  SOCK_Socket::PutStrings -
*
* DESCRIPTION:
*
* RETURN:
*
* INTERFACE NOTE:
*
* History:
*   2001.06.25 ganda: Creation
**********************************************************************/

Status
SOCK_Socket::PutStrings(char *str, int len)
{
  if (send(sinFdCv, str, len,0) > 0){
    return (OK);
  }  else  {
    return (ERROR);
  }
}
